@@ -24,6 +24,7 @@ def compute_metrics_for_binary_outcome(
     predictions: np.ndarray,
     threshold_positive: Union[None, str, float] = 0.5,
     share_positive: Union[None, str, float] = None,
+    weights: Union[None, np.ndarray] = None,
 ) -> OrderedDict:
     """Evaluate predicted probabilities against actual binary outcome values.
 
@@ -41,6 +42,8 @@ def compute_metrics_for_binary_outcome(
             the predicted share positive in each time horizon. Probability ties
             may cause share of positives in output to exceed given value.
             Overrides threshold_positive.
+        weights: A 1-D array of weights with the same length as actuals and predictions.
+        Each prediction contributes to metrics in proportion to its weight. If None, each prediction has the same weight.
 
     Returns:
         An ordered dictionary containing key-value pairs for area under the
@@ -50,11 +53,11 @@ def compute_metrics_for_binary_outcome(
     """
     metrics = OrderedDict()
     if actuals.any() and not actuals.all():
-        metrics["AUROC"] = roc_auc_score(actuals, predictions)
+        metrics["AUROC"] = roc_auc_score(actuals, predictions, sample_weight=weights)
     else:
         metrics["AUROC"] = np.nan
-    metrics["Predicted Share"] = predictions.mean()
-    metrics["Actual Share"] = actuals.mean()
+    metrics["Predicted Share"] = np.average(predictions, weights=weights)
+    metrics["Actual Share"] = np.average(actuals, weights=weights)
     if actuals.empty:
         (
             metrics["True Positives"],
@@ -64,11 +67,11 @@ def compute_metrics_for_binary_outcome(
         ) = [0, 0, 0, 0]
     else:
         if share_positive == "predicted":
-            share_positive = predictions.mean()
+            share_positive = metrics["Predicted Share"]
         if share_positive is not None:
             threshold_positive = np.quantile(predictions, 1 - share_positive)
         elif threshold_positive == "predicted":
-            threshold_positive = predictions.mean()
+            threshold_positive = metrics["Predicted Share"]
         (
             metrics["True Positives"],
             metrics["False Negatives"],
@@ -76,7 +79,10 @@ def compute_metrics_for_binary_outcome(
             metrics["True Negatives"],
         ) = (
             confusion_matrix(
-                actuals, predictions >= threshold_positive, labels=[True, False]
+                actuals,
+                predictions >= threshold_positive,
+                labels=[True, False],
+                sample_weight=weights,
             )
             .ravel()
             .tolist()
@@ -85,7 +91,9 @@ def compute_metrics_for_binary_outcome(
 
 
 def compute_metrics_for_categorical_outcome(
-    actuals: pd.DataFrame, predictions: np.ndarray
+    actuals: pd.DataFrame,
+    predictions: np.ndarray,
+    weights: Union[None, np.ndarray] = None,
 ) -> OrderedDict:
     """Evaluate predicted probabilities against actual categorical outcome values.
 
@@ -93,21 +101,32 @@ def compute_metrics_for_categorical_outcome(
         actuals: A DataFrame representing one-hot-encoded actual class membership.
         predictions: A DataFrame of predicted probabilities of class membership
             for the respective observations represented in actuals.
+        weights: A 1-D array of weights with the same length as actuals and predictions.
+        Each prediction contributes to metrics in proportion to its weight. If None, each prediction has the same weight.
 
     Returns:
         An ordered dictionary containing a key-value pair for area under the
-        receiver operating characteristic curve (AUROC).
+        receiver operating characteristic curve (AUROC). Classes with no positive values are not included in AUROC calculation.
     """
     metrics = OrderedDict()
-    if (actuals.all() | ~actuals.any()).any():
+    if actuals.all().any():
         metrics["AUROC"] = np.nan
     else:
-        metrics["AUROC"] = roc_auc_score(actuals, predictions, multi_class="ovr")
+        positive_cols = actuals.any()
+        metrics["AUROC"] = roc_auc_score(
+            actuals.loc[:, positive_cols],
+            predictions[:, positive_cols],
+            multi_class="ovr",
+            average="weighted",
+            sample_weight=weights,
+        )
     return metrics
 
 
 def compute_metrics_for_numeric_outcome(
-    actuals: pd.core.series.Series, predictions: np.ndarray
+    actuals: pd.core.series.Series,
+    predictions: np.ndarray,
+    weights: Union[None, np.ndarray] = None,
 ) -> OrderedDict:
     """Evaluate predicted numeric values against actual outcome values.
 
@@ -119,7 +138,7 @@ def compute_metrics_for_numeric_outcome(
         An ordered dictionary containing a key-value pair for R-squared.
     """
     metrics = OrderedDict()
-    metrics["R-squared"] = r2_score(actuals, predictions)
+    metrics["R-squared"] = r2_score(actuals, predictions, sample_weight=weights)
     return metrics
 
 
@@ -145,6 +164,9 @@ class Modeler(ABC):
             periods since the earliest period in the data.
         max_lead_col (str): Name of the column representing the number of
             observable future periods.
+        spell_col (str): Name of the column representing the number of
+            previous spells of consecutive observations of the same individual.
+        weight_col (str): Name of the column representing observation weights.
         reserved_cols (list): Column names of non-features.
         numeric_features (list): Column names of numeric features.
         n_intervals (int): The largest number of periods ahead to forecast
@@ -166,6 +188,7 @@ class Modeler(ABC):
         period_col: str = "_period",
         max_lead_col: str = "_maximum_lead",
         spell_col: str = "_spell",
+        weight_col: Union[None, str] = None,
         allow_gaps: bool = False,
     ) -> None:
         """Characterize data for modelling.
@@ -193,6 +216,7 @@ class Modeler(ABC):
                 observable future periods.
             spell_col (str): Name of the column representing the number of
                 previous spells of consecutive observations of the same individual.
+            weight_col (str): Name of the column representing observation weights.
             allow_gaps (bool): Whether or not observations should be included for
                 training and evaluation if there is a period without an observation
                 between the period of observations and the last period of the
@@ -220,6 +244,7 @@ class Modeler(ABC):
         self.period_col = period_col
         self.max_lead_col = max_lead_col
         self.spell_col = spell_col
+        self.weight_col = weight_col
         self.allow_gaps = allow_gaps
         self.reserved_cols = [
             self.duration_col,
@@ -233,6 +258,8 @@ class Modeler(ABC):
         ]
         if self.config:
             self.reserved_cols.append(self.config["INDIVIDUAL_IDENTIFIER"])
+        if self.weight_col:
+            self.reserved_cols.append(self.weight_col)
         if self.data is not None:
             self.categorical_features = [
                 col for col in self.data.select_dtypes("category")
@@ -341,6 +368,7 @@ class SurvivalModeler(Modeler):
             observable future periods.
         spell_col (str): Name of the column representing the number of
             previous spells of consecutive observations of the same individual.
+        weight_col (str): Name of the column representing observation weights.
         reserved_cols (list): Column names of non-features.
         numeric_features (list): Column names of numeric features.
         n_intervals (int): The largest number of periods ahead to forecast.
@@ -400,19 +428,22 @@ class SurvivalModeler(Modeler):
         lead_lengths = np.arange(self.n_intervals) + 1
         for lead_length in lead_lengths:
             actuals = self.label_data(lead_length - 1)[subset].reset_index()
-            actuals = actuals[actuals[self.max_lead_col] >= lead_length]["label"]
+            actuals = actuals[actuals[self.max_lead_col] >= lead_length]
+            weights = actuals[self.weight_col].values if self.weight_col else None
+            actuals = actuals["label"]
             metrics.append(
                 compute_metrics_for_binary_outcome(
                     actuals,
                     predictions[:, lead_length - 1][actuals.index],
                     threshold_positive=threshold_positive,
                     share_positive=share_positive,
+                    weights=weights,
                 )
             )
         metrics = pd.DataFrame(metrics, index=lead_lengths)
         metrics.index.name = "Lead Length"
         metrics["Other Metrics:"] = ""
-        if not self.allow_gaps:
+        if (not self.allow_gaps) and (self.weight_col is None):
             concordance_index_value = concordance_index(
                 self.data[subset][[self.duration_col, self.max_lead_col]].min(axis=1),
                 np.sum(predictions, axis=-1),
@@ -597,6 +628,7 @@ class StateModeler(Modeler):
             observable future periods.
         spell_col (str): Name of the column representing the number of
             previous spells of consecutive observations of the same individual.
+        weight_col (str): Name of the column representing observation weights.
         reserved_cols (list): Column names of non-features.
         numeric_features (list): Column names of numeric features.
         n_intervals (int): The largest number of periods ahead to forecast.
@@ -653,7 +685,9 @@ class StateModeler(Modeler):
         for lead_length in lead_lengths:
             actuals = self.subset_for_training_horizon(
                 self.label_data(lead_length - 1)[subset].reset_index(), lead_length - 1
-            )["label"]
+            )
+            weights = actuals[self.weight_col].values if self.weight_col else None
+            actuals = actuals["label"]
             if self.objective == "multiclass":
                 actuals = pd.DataFrame(
                     {label: actuals == label for label in range(self.num_class)},
@@ -663,6 +697,7 @@ class StateModeler(Modeler):
                     compute_metrics_for_categorical_outcome(
                         actuals,
                         predictions[:, :, lead_length - 1].T[actuals.index],
+                        weights=weights,
                     )
                 )
             else:
@@ -670,6 +705,7 @@ class StateModeler(Modeler):
                     compute_metrics_for_numeric_outcome(
                         actuals,
                         predictions[:, lead_length - 1][actuals.index],
+                        weights=weights,
                     )
                 )
         metrics = pd.DataFrame(metrics, index=lead_lengths)
@@ -787,7 +823,9 @@ class ExitModeler(StateModeler):
             data = data[data["exit"].isna() & (data[self.max_lead_col] > time_horizon)]
             data = data.drop("exit", axis=1)
         else:
-            data = [data[self.event_col] & (data[self.duration_col] == time_horizon)]
+            data = data[
+                data[self.event_col] & (data[self.duration_col] == time_horizon)
+            ]
         return data
 
     def label_data(self, time_horizon: int) -> pd.Series:
@@ -796,7 +834,12 @@ class ExitModeler(StateModeler):
         data[self.duration_col] = data[[self.duration_col, self.max_lead_col]].min(
             axis=1
         )
+        if self.objective == "multiclass":
+            temp_exit_col = data[self.exit_col]
+            data[self.exit_col] = data[self.exit_col].cat.codes
         data["label"] = data.groupby(
             [self.config["INDIVIDUAL_IDENTIFIER"], self.spell_col]
         )[self.exit_col].transform("last")
+        if self.objective == "multiclass":
+            data[self.exit_col] = temp_exit_col
         return data
