@@ -43,6 +43,7 @@ class LGBModeler(Modeler):
             observable future periods.
         spell_col (str): Name of the column representing the number of
             previous spells of consecutive observations of the same individual.
+        weight_col (str): Name of the column representing observation weights.
         reserved_cols (list): Column names of non-features.
         numeric_features (list): Column names of numeric features.
         n_intervals (int): The largest number of periods ahead to forecast.
@@ -86,12 +87,14 @@ class LGBModeler(Modeler):
             """Compute out-of-sample performance for a parameter set."""
             params = {}
             params["num_iterations"] = trial.suggest_int("num_iterations", 8, 128)
-            params["learning_rate"] = trial.suggest_uniform("learning_rate", 2e-5, 0.5)
+            params["learning_rate"] = trial.suggest_uniform(
+                "learning_rate", 2 ** -5, 0.5
+            )
             params["num_leaves"] = trial.suggest_int("num_leaves", 8, 256)
             params["max_depth"] = trial.suggest_int("max_depth", 4, 32)
             params["min_data_in_leaf"] = trial.suggest_int("min_data_in_leaf", 4, 512)
             params["min_sum_hessian_in_leaf"] = trial.suggest_uniform(
-                "min_sum_hessian_in_leaf", 2e-5, 0.25
+                "min_sum_hessian_in_leaf", 2 ** -5, 0.25
             )
             params["bagging_freq"] = trial.suggest_int("bagging_freq", 0, 1)
             params["bagging_fraction"] = trial.suggest_uniform(
@@ -171,12 +174,18 @@ class LGBModeler(Modeler):
                     self.categorical_features + self.numeric_features
                 ],
                 label=data[~validation_subset]["label"],
+                weight=data[~validation_subset][self.weight_col]
+                if self.weight_col
+                else None,
             )
             validation_data = train_data.create_valid(
                 data[validation_subset][
                     self.categorical_features + self.numeric_features
                 ],
                 label=data[validation_subset]["label"],
+                weight=data[validation_subset][self.weight_col]
+                if self.weight_col
+                else None,
             )
             study = optuna.create_study(
                 pruner=optuna.pruners.MedianPruner(),
@@ -274,12 +283,18 @@ class LGBModeler(Modeler):
                     self.categorical_features + self.numeric_features
                 ],
                 label=data[~data[self.validation_col]]["label"],
+                weight=data[~data[self.validation_col]][self.weight_col]
+                if self.weight_col
+                else None,
             )
             validation_data = train_data.create_valid(
                 data[data[self.validation_col]][
                     self.categorical_features + self.numeric_features
                 ],
                 label=data[data[self.validation_col]]["label"],
+                weight=data[data[self.validation_col]][self.weight_col]
+                if self.weight_col
+                else None,
             )
             model = lgb.train(
                 params[time_horizon],
@@ -294,6 +309,7 @@ class LGBModeler(Modeler):
             data = lgb.Dataset(
                 data[self.categorical_features + self.numeric_features],
                 label=data["label"],
+                weight=data[self.weight_col] if self.weight_col else None,
             )
             model = lgb.train(
                 params[time_horizon],
@@ -338,7 +354,12 @@ class LGBModeler(Modeler):
         """Transform features to suit model training."""
         data = self.data.copy(deep=True)
         if self.config.get("DATETIME_AS_DATE", True):
-            for col in data.select_dtypes("datetime"):
+            date_cols = data.select_dtypes("datetime").columns + [
+                col
+                for col in data.select_dtypes("category")
+                if np.issubdtype(data[col].cat.categories.dtype, np.datetime64)
+            ]
+            for col in date_cols:
                 data[col] = (
                     data[col].dt.year * 10000
                     + data[col].dt.month * 100
@@ -348,6 +369,9 @@ class LGBModeler(Modeler):
             data[data.select_dtypes("datetime")] = pd.to_numeric(
                 data[data.select_dtypes("datetime")]
             )
+            for col in data.select_dtypes("category"):
+                if np.issubdtype(data[col].cat.categories.dtype, np.datetime64):
+                    data[col] = data[col].astype(int)
         return data
 
     def save_model(self, file_name: str = "GBT_Model", path: str = "") -> None:
